Attribute VB_Name = "modMain"
Option Explicit

Public Enum IconAction
    iaAdd = 0&
    iaModify = 1&
    iaDelete = 2&
End Enum

Public Enum IconMember
    imCallback = 1&
    imIcon = 2&
    imHint = 4&
End Enum

Public Type IconData
  idSize                    As Long
  idHwnd                    As Long
  idID                      As Long
  idFlags                   As Long
  idCallBackID              As Long
  idIcon                    As Long
  idTip                     As String * 64
End Type

Public Enum WindowChange
    wcCreated = 0&
    wcDestroyed = 1&
End Enum

Public Enum SubClassAction
    saInit = 0&
    saDestroy = 1&
End Enum

Public Const APP_ID         As Long = 1024&
Public Const CALLBACK_ID    As Long = 1440&
Public Const GWL_WNDPROC    As Long = -4&
Public Const GWL_STYLE      As Long = -16&
Public Const WS_BORDER      As Long = 8388608

Public Declare Function EnumWindows Lib "user32" (ByVal lpEnumFunc As Long, ByVal lParam As Long) As Long
Public Declare Function EnumChildWindows Lib "user32" (ByVal hWndParent As Long, ByVal lpEnumFunc As Long, ByVal lParam As Long) As Long
Public Declare Function GetClassName Lib "user32" Alias "GetClassNameA" (ByVal hWnd As Long, ByVal lpClassName As String, ByVal nMaxCount As Long) As Long
Public Declare Function GetWindowLong Lib "user32" Alias "GetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As Long) As Long
Public Declare Function ShowWindow Lib "user32" (ByVal hWnd As Long, ByVal nCmdShow As Long) As Long
Public Declare Function SysTrayProc Lib "shell32.dll" Alias "Shell_NotifyIconA" (ByVal dwMessage As Long, lpData As IconData) As Long

Private Declare Function CallWindowProc Lib "user32" Alias "CallWindowProcA" (ByVal lpPrevWndFunc As Long, ByVal hWnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Private Declare Function GetWindowText Lib "user32" Alias "GetWindowTextA" (ByVal hWnd As Long, ByVal lpString As String, ByVal cch As Long) As Long
Private Declare Function GetWindowTextLength Lib "user32" Alias "GetWindowTextLengthA" (ByVal hWnd As Long) As Long
Private Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As Long, ByVal dwNewLong As Any) As Long

Public pobjWindows          As IWindows
Public pobjSysTray          As ISysTray

Private mlngPrevProcID      As Long

Public Function EnumWindowsProc(ByVal hWnd As Long, ByVal lParam As Long) As Long
    On Error Resume Next
    Dim strWindowText       As String
    Dim strClassName        As String
    GetWindowNames hWnd, strWindowText, strClassName
    pobjWindows.Add hWnd, strWindowText, strClassName
    EnumWindowsProc = 1
End Function

Public Function EnumChildWindowsProc(ByVal hWnd As Long, ByVal lParam As Long) As Long
    On Error Resume Next
    Dim lngHwndParent       As Long
    Dim strWindowText       As String
    Dim strClassName        As String
    GetWindowNames hWnd, strWindowText, strClassName
    pobjWindows.Add hWnd, strWindowText, strClassName
    If lParam Then
        pobjWindows(lParam).Add hWnd, strWindowText, strClassName, lParam
    End If
    EnumChildWindowsProc = 1
End Function

Private Function Intercept(ByVal hWnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
    On Error Resume Next
    If mlngPrevProcID = 0 Then Exit Function
    If uMsg = CALLBACK_ID Then
        Select Case lParam
            Case meMove, _
                 meLeftDblClk, meLeftDown, meLeftUp, _
                 meRightDblClk, meRightDown, meRightUp, _
                 meMidDblClk, meMidDown, meMidUp
                 pobjSysTray.ForwardEvent lParam
        End Select
    End If
    Intercept = CallWindowProc(mlngPrevProcID, hWnd, uMsg, wParam, lParam)
End Function

Public Function SubClass(ByVal hWnd As Long, Optional ByVal saAction As SubClassAction) As Boolean
    On Error Resume Next
    Select Case saAction
        Case saInit
            If mlngPrevProcID Then Exit Function
            mlngPrevProcID = SetWindowLong(hWnd, GWL_WNDPROC, AddressOf Intercept)
            SubClass = mlngPrevProcID
        Case saDestroy
            If mlngPrevProcID = 0 Then Exit Function
            SetWindowLong hWnd, GWL_WNDPROC, mlngPrevProcID
            mlngPrevProcID = 0
            SubClass = True
    End Select
End Function

Public Sub GetWindowNames(ByVal hWnd As Long, Optional ByRef strWindowTitle As String, Optional ByRef strClassTitle As String)
    Dim strWinText          As String
    Dim strClassText        As String
    strWinText = Space(256&)
    strClassText = Space(256&)
    GetWindowText hWnd, ByVal strWinText, Len(strWinText)
    GetClassName hWnd, ByVal strClassText, Len(strClassText)
    strWindowTitle = Trim$(Replace(strWinText, Chr(0), vbNullString))
    strClassTitle = Trim$(Replace(strClassText, Chr(0), vbNullString))
End Sub
